//***********************************
//coding for variables of the dataset
//***********************************
function [descriptors,target]= recodage(D)
    //transforming the columns 1 to 9 in numeric
    descriptors = evstr(D(:,1:9))
    //getting the class attribute
    y=D(:,10:10)
    //"begnin" -> 1 and "malignant" -> 2
    //we could have used the predefined nan_grp2idx() function here
    n=size(y,1) //n = number of instances in the dataset
    grp=zeros(n,1) //creating a vector of zeros
    for i=1:n
        if y(i)=="begnin" then
            grp(i)=1
        else
            grp(i)=2
        end
    end
    target=grp
endfunction

//*******************************************
//function for the evaluation of a classifier
//*******************************************
function [MC,ERR_RATE] = test_classifier(classifier,descriptors,target)
    //prediction
    pred=nan_test_sc(classifier,descriptors)
    //confusion matrix
    MC=nan_confusionmat(target,pred.classlabel)
    //error rate
    ERR_RATE = 1.0 - sum(diag(MC))/sum(MC)
endfunction

//loading the "Nan" toolbox
atomsLoad("nan")

//loading the learning set - "string" format
DTrain=csvRead("breast-train.txt","\t",".","string",[],[],[2 1 400 10])

//calling the function for the data preparation
[XTrain,CTrain]=recodage(DTrain)
disp("Classes distribution - Learning set")
disp(tabul(CTrain))

//learning phase - LD2 (linear discriminant analysis) method
modelLD2=nan_train_sc(XTrain,CTrain,'LD2')
disp("LD2 from the nan library")
disp(modelLD2)
disp("Weights form LD2")
disp(modelLD2.weights)

//loading the test set - "string" format
DTest=csvRead("breast-test.txt","\t",".","string",[],[],[2 1 301 10])

//calling the function for the data preparation
[XTest,CTest]=recodage(DTest)
disp("Classes distribution - Test set")
disp(tabul(CTest))

//évaluation
[mc,err_rate] = test_classifier(modelLD2,XTest,CTest)
disp("Confusion matrix - Test set")
disp(mc)
disp("Error rate - Test set")
disp(err_rate)

//learning and testing for the Naive Bayes Classifier approach
modelNBC=nan_train_sc(XTrain,CTrain,'NBC')
disp("Naive Bayes Classifier from the nan library")
disp(modelNBC)
disp(modelNBC.MEAN)
disp(modelNBC.VAR)
//evaluation
[mc,err_rate] = test_classifier(modelNBC,XTest,CTest)
disp("Confusion matrix - Test set")
disp(mc)
disp("Error rate - Test set")
disp(err_rate)

//loading the "libsvm" toolbox
atomsLoad('libsvm')

//*** SVM with RBF kernel ***
//learning phase
modelSVM=libsvm_svmtrain(CTrain,XTrain)
disp("SVM form libsvm - RBF kernel")
disp(modelSVM)
//testing phase
[predSVM]=libsvm_svmpredict(CTest,XTest,modelSVM)
mc=libsvm_confmat(CTest,predSVM)
disp("Confusion matrix - Test set")
disp(mc)

//*** SVM with linear kernel ***
//learning phase
modelLINSVM=libsvm_svmtrain(CTrain,XTrain,'-t 0')
disp("SVM form libsvm - Linear kernel")
disp(modelLINSVM)
//testing phase
[predLINSVM]=libsvm_svmpredict(CTest,XTest,modelLINSVM)
mc=libsvm_confmat(CTest,predLINSVM)
disp("Confusion matrix - Test set")
disp(mc)

//more détails abou the output of the linear SVM
//support vector coefficients
disp(modelLINSVM.sv_coef')
//support vector coordinates (5 first support vectors)
disp(modelLINSVM.SVs(1:5,:))
